#ifdef Py_ARRAYOBJECT_H
#error "Can't use both NumPy and numpy_compat in same module."
#endif

#if !defined(__arrayobject_h)
#define __arrayobject_h

#define REFCOUNT(obj) (((PyObject *)(obj))->ob_refcnt)
#define MAX_ELSIZE 16

#include "libnumarray.h"

enum PyArray_TYPES { 
 #if defined(USE_SIGNED_CHAR)
	PyArray_CHAR=tInt8, 
 #else
	PyArray_CHAR=tUInt8,
 #endif
	PyArray_UBYTE=tUInt8, 
	PyArray_SBYTE=tInt8,
	PyArray_SHORT=tInt16, 
	PyArray_INT=tInt32, 
	PyArray_LONG=tLong,
	PyArray_FLOAT=tFloat32, 
	PyArray_DOUBLE=tFloat64, 
	PyArray_CFLOAT=tComplex32, 
	PyArray_CDOUBLE=tComplex64,
	PyArray_NOTYPE=tAny
};

/* Array flags */
#define CONTIGUOUS 1                        /* compatible, depends */
#define OWN_DIMENSIONS 2                    /* always false */
#define OWN_STRIDES 4                       /* always false */
#define OWN_DATA 8                          /* always false */

/* Original NumPy Descr
typedef struct {
  PyArray_VectorUnaryFunc *cast[PyArray_NTYPES]; 
  PyArray_GetItemFunc *getitem;
  PyArray_SetItemFunc *setitem;
  int type_num, elsize;
  char *one, *zero;
  char type;
} PyArray_Descr;
*/

/* numpy_compat PyArray_Descr */
typedef struct {
	int  type_num;
	int  elsize;
	char type;    /* One of "cb1silfdFD "  Object arrays not supported. */
} PyArray_Descr;

/* numpy_compat PyArrayObject */
typedef struct {
	PyObject_HEAD
	char *data;
	int nd;
	int dimensions[MAXDIM];
	int strides[MAXDIM];
	PyObject *base;
	PyArray_Descr descr[1];
	int flags;
} PyArrayObject;       


#include "numpy_compat.h"   /* Numeric compatability functions */

#define import_array() import_libnumarray(); import_numpy_compat()

#define PyArray_SIZE(mp)        PyArray_Size(mp)
#define PyArray_NBYTES(mp)      ((mp)->descr->elsize * PyArray_Size(mp))
#define PyArray_ISCONTIGUOUS(m) ((m)->flags & CONTIGUOUS)
#define PyArray_ISSPACESAVER(m) 0

#endif
